package it.demo.prove;

//Classi definite da noi
import it.demo.dominio.Dipartimento;
import it.demo.util.HibernateUtil;

//Classi di Hibernate
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.HibernateException;
import org.hibernate.Query;

//Classi generiche
import java.util.List;


/**Classe che fa vedere come usare Hibernate 
 * @author Leonardo Puleggi
 *
 */
public class ProvaEsempio_01 {
	
	/**Restituisce una lista di tutti i dipartimenti salvati nel database
	 * 
	 * @return lista dei Dipartimenti del database
	 */
	@SuppressWarnings("unchecked")
	public static List<Dipartimento> listaTuttiDipartimenti() {
		List<Dipartimento> result=null;
		
		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			Query q = session.createQuery("from Dipartimento");
			result=q.list();
			tx.commit();
		}
		catch (HibernateException he) {
			if (tx!=null) 
				tx.rollback();
			throw he;
		}
		finally {
			session.close();
		}
		
		return result;
	}
	
	/**Crea un oggetto a partire dall'identificativo o solleva un'eccezione
	 * 
	 */
	public static Dipartimento cercaDipartimento(long id) {
		Dipartimento result=null;
		
		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			result=(Dipartimento)session.load(Dipartimento.class,id);
			tx.commit();
		}
		catch (HibernateException he) {
			if (tx!=null) 
				tx.rollback();
			System.out.println("Non e' stato trovato l'oggetto con questo id");
		}
		finally {
			session.close();
		}
		return result;
	}

	/**Crea un Dipartimento dati nome e sede e lo salva nel database
	*/
	public static Dipartimento creaDipartimento(String nome,String sede) {
		//Att non specifico l'id lo...creer lui da solo
		Dipartimento d=new Dipartimento(nome,sede);
		
		
		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			session.persist(d);
			tx.commit();
		}
		catch (HibernateException he) {
			if (tx!=null) 
				tx.rollback();
			throw he;
		}
		finally {
			session.close();
		}
		return d;
	}

	/**Esegue tutti i metodi di prova
	 * 
	 */
	public static void main(String[] args) {
		System.out.println("Stampo tutti i Dipartimenti");
		for(Dipartimento d : listaTuttiDipartimenti())
			System.out.println(d.toString());
		System.out.println("\nOra stampo solo quello che ha id=2  (Metodo classico)");
		System.out.println(cercaDipartimento(2).toString());
		System.out.println("\nOra stampo solo quello che ha id=18(dovrebbe essere null)");
		if(cercaDipartimento(18)==null)
			System.out.println("Infatti e' nullo");
		else
			System.out.println("O  stato inserito per una prova o c' un errore");

		System.out.println("\nOra creo un Dipartimento con nome leo e sede roma");
		System.out.println(creaDipartimento("leo","roma").toString());
		System.out.println("Notare che l'id e' stato generato automaticamente");
		
	}
}