package it.demo.prove;

//Classi definite da noi
import it.demo.dominio.Dipartimento;
import it.demo.dominio.Persona;
import it.demo.util.HibernateUtil;

//Classi di Hibernate
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.HibernateException;
import org.hibernate.Query;

//Classi generiche
import java.util.List;


/**Classe che fa vedere come usare Hibernate 
 * @author Leonardo Puleggi
 *
 */
public class ProvaEsempio_03 {
	
	/**Restituisce una lista di tutti i dipartimenti salvati nel database
	 * 
	 * @return lista dei Dipartimenti del database
	 */
	@SuppressWarnings("unchecked")
	public static List<Dipartimento> listaTuttiDipartimenti() {
		List<Dipartimento> result=null;
		
		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			Query q = session.createQuery("from Dipartimento");
			result=q.list();
			tx.commit();
		}
		catch (HibernateException he) {
			if (tx!=null) 
				tx.rollback();
			throw he;
		}
		finally {
			session.close();
		}
		
		return result;
	}


	/**Metodo per far vedere come sia possibile usare due transizioni diverse
	 * 
	 */
	public static void dueSessioni() {
		//Creo il dipartimento e lo salvo
		Dipartimento d=creaDipartimento("leo","roma");
		
		//ipotizziamo che l'oggetto venga passato allo strato superiore
		//e che la sesssione sia chiusa
		try {
			System.out.print("Ora sto fermo per 2 secondi per far finta che l'oggetto subisca qualche elaborazione\n\n");
			Thread.sleep(2000);
		}
		catch (Exception e) {
		}
		
		d.setNome("programmazione 3");
		/*Ora salvo notare che l'unica istruzione che ci interessa  il savorUpdate*/
		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			session.saveOrUpdate(d);
			tx.commit();
		}
		catch (HibernateException he) {
			if (tx!=null) 
				tx.rollback();
			throw he;
		}
		finally {
			session.close();
		}
	}
	
	private static Dipartimento creaDipartimento(String nome,String sede) {
		//Att non specifico l'id lo...creer lui da solo
		Dipartimento d=new Dipartimento(nome,sede);
		
		
		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			session.persist(d);
			tx.commit();
		}
		catch (HibernateException he) {
			if (tx!=null) 
				tx.rollback();
			throw he;
		}
		finally {
			session.close();
		}
		return d;
	}

	/**Esegue tutti i metodi di prova
	 * 
	 */
	public static void main(String[] args) {

		Dipartimento d=listaTuttiDipartimenti().iterator().next();
		System.out.println("Stampo la lista delle persone del primo dipartimento");
		System.out.println(d.getPersone().toString());

		System.out.print("\nDato il primo dipartimento prendo la prima persona che vi lavora e verifico che questa ");
		System.out.print("persona abbia il riferimento corretto al dipartimento(cioe' la navigabilita' e' bidirezionale)\n\n");
		System.out.println("La prima persona e' "+((Persona)d.getPersone().iterator().next()).getNome());
		System.out.println("Il nome del dipartimento della prima persona: "+((Persona)d.getPersone().iterator().next()).getDipartimento().getNome());
		System.out.println("La sede del dipartimento della prima persona: "+((Persona)d.getPersone().iterator().next()).getDipartimento().getSede());

		System.out.print("\n\nOra facciamo la prova delle due sessioni, nella prima creo un oggetto Dipartimento e lo salvo nel database, ");
		System.out.print(" faccio finta di eseguire operazioni in uno strato superiore e poi chiedo di aggiornarlo\n\n");
		dueSessioni();
		System.out.println("Verifichiamo che esista il Dipartimento 'programmazione 3' con sede a roma dalla lista di tutti i dipartimenti");
		for (Dipartimento dip : listaTuttiDipartimenti())
			System.out.println("'"+dip.getNome()+"' "+dip.getSede());
	}
}