package it.demo.prove;

//Classi definite da noi
import it.demo.dominio.Dipartimento;
import it.demo.dominio.Persona;
import it.demo.util.HibernateUtil;

//Classi di Hibernate
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.HibernateException;
import org.hibernate.Query;

//Classi generiche
import java.util.List;

/**Classe che fa vedere come usare Hibernate 
 * @author Leonardo Puleggi
 *
 */
public class ProvaEsempio_04 {
	
	/**Restituisce una lista di tutti i dipartimenti salvati nel database
	 * 
	 * @return lista dei Dipartimenti del database
	 */
	@SuppressWarnings("unchecked")
	public static List<Dipartimento> listaTuttiDipartimenti() {
		List<Dipartimento> result=null;
		
		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			Query q = session.createQuery("from Dipartimento");
			result=q.list();
			tx.commit();
		}
		catch (HibernateException he) {
			if (tx!=null) 
				tx.rollback();
			throw he;
		}
		finally {
			session.close();
		}
		
		return result;
	}

	@SuppressWarnings("unchecked")
	public static Dipartimento creaDipartimento(String nome,String sede) {
		Dipartimento d=null;	
		
		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			List<Dipartimento> l=session.createQuery("from Dipartimento where nome=:nome and sede=:sede")
			.setString("nome",nome).setString("sede",sede).list();
			if (l.size()==0) {
				d=new Dipartimento(nome,sede);
				session.save(d);
			}
			else
				d=l.iterator().next();
				
			tx.commit();
		}
		catch (HibernateException he) {
			if (tx!=null) 
				tx.rollback();
			throw he;
		}
		finally {
			session.close();
		}
		return d;
	}
	
	@SuppressWarnings("unchecked")
	public static Dipartimento duplicati() {
		Dipartimento d=null;
		
		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			d=(Dipartimento)session.createQuery("from Dipartimento where id_dipartimento=1").uniqueResult();
			System.out.println("Il dipartimento che carico e':\n"+d.toString());
			
			Persona p1=new Persona("prova");
			Persona p2=new Persona("leonardo");
			p1.setDipartimento(d);
			p2.setDipartimento(d);
			d.aggiugiPersona(p1);
			d.aggiugiPersona(p2);
			session.saveOrUpdate(d);
			tx.commit();
		}
		catch (HibernateException he) {
			if (tx!=null) 
				tx.rollback();
			throw he;
		}
		finally {
			session.close();
		}
		return d;
	}

	/**Esegue tutti i metodi di prova
	 * 
	 */
	public static void main(String[] args) {
	
		System.out.println("Provo ad inserire due dipartimenti uguali");
		creaDipartimento("programmazione 4","roma");
		creaDipartimento("programmazione 4","roma");
		System.out.println("Finito..verifichiamo che ce ne sia solo uno stampando la lista dei dipartimenti");
		for (Dipartimento d: listaTuttiDipartimenti())
			System.out.println(d.getNome()+" "+d.getSede());
		
		System.out.println("\n\nOra proviamo a caricare un Dipartimento,ad aggiornare con altre due persone" +
				"(di cui una gi presente nel database) e faccio l'update");
		Dipartimento dip=duplicati();
		System.out.println(dip.toString());
	}
}